/*******************************************************************************

                                TZlib beta

                   Copyright (c) 1998 by Richard W. Frost

This file contains the declarations for the TZlib class, a C++ wrapper for the
'zlib' compression and decompression engine.  This class uses the zlib general
purpose compression library version 1.0.4, Jul 24th, 1996 Copyright (C)
1995-1996 by Jean-loup Gailly and Mark Adler.  For more information about zlib
see the zlib.h header file or visit the zlib home page at:

					http://www.cdrom.com/pub/infozip/zlib/

The TZlib class is a C++Builder adaptation and expansion of the freeware C++
ZlibEngine class.  ZlibEngine is a Tiny Software (tm) project written by Mark
Nelson.  Contact markn@tiny.com for more information. Or visit his web site at:

                http://web2.airmail.net/markn/index.html


TZlib has the following public interface:

- int TZlib::Compress(TStream *input, TStream *output, int level)
   Compress is the function used to compress a single data stream. It takes
   three arguments: the input stream, the output stream, and the compression
   level.  Thanks to the miracle of polymorphism, the input and output streams
   can be any of the descendants of the TStream class such as TFileStream,
   TMemoryStream, TBlobStream, etc., or a combination.  See Decompress below for
   return values.

   The compression level can be Z_DEFAULT_COMPRESSION, or between 0 and 9:
   1 gives best speed, 9 gives best compression, 0 gives no compression at all
   (the input data is simply copied a block at a time).  Using
   Z_DEFAULT_COMPRESSION requests a default compromise between speed and
   compression (currently equivalent to level 6).

- int TZlib::Decompress(TStream *input, TStream *output)
   Decompress is the function used to decompress a single data stream. It takes
   only two arguments: the input stream to decompress and the output stream.
   There is no level parameter like in Compress, because the zlib delate
   algorithm doesn't need it.

   Compress and Decompress both return the ZlibError as a result. ZlibError can
   be any of the following:

                        #define Z_OK              0
                        #define Z_STREAM_END      1
                        #define Z_NEED_DICT       2
                        #define Z_ERRNO         (-1)
                        #define Z_STREAM_ERROR  (-2)
                        #define Z_DATA_ERROR    (-3)
                        #define Z_MEM_ERROR     (-4)
                        #define Z_BUF_ERROR     (-5)
                        #define Z_VERSION_ERROR (-6)
                        #define Z_USER_ABORT    (-7)

   The functions return Z_OK if success, Z_STREAM_END if all input has been
   consumed and all output has been produced, Z_NEED_DICT if a preset dictionary
   is needed at this point (see zlib.h inflateSetDictionary for details),
   Z_STREAM_ERROR if the stream state was inconsistent or if level is not a
   valid compression level, Z_DATA_ERROR if the stream was freed prematurely
   (some input or output was discarded) or if the input data was corrupted,
   Z_MEM_ERROR if there was not enough memory, Z_BUF_ERROR if no progress is
   possible or there was not enough room in the output buffer, Z_VERSION_ERROR
   if the zlib library version (zlib_version) is incompatible with the version
   assumed by the caller (ZLIB_VERSION).

- __property bool UserAbort
   Setting UserAbort to true will abort the compression or decompression
   process.  The Windows message loop is churned during both compression and
   decompression so that the process can be aborted by a button press or other
   windows event.

-  typedef void __fastcall (__closure *TZlibProgressEvent)
                     (System::TObject* Sender, int Percent);
   __property TZlibProgressEvent OnZlibProgress
    The OnZlibProgress event is triggered by the Compress and Decompress loops.
    The percent value is provided so that you can use the OnZlibProgress event
    to update a progress bar or other progress type display.

-  typedef void __fastcall (__closure *TZlibResultEvent)
                     (System::TObject* Sender, AnsiString message, int zresult);
   __property TZlibResultEvent OnZlibResult
    The OnZlibResult event is triggered after each call to the Compress and
    Decompress functions.  OnZlibResult provides the user with a status message
    and the ZlibError result. (See ZlibError above under Decompress.)

*******************************************************************************/

//---------------------------------------------------------------------------
#ifndef TZlibH
#define TZlibH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
// All of the zlib code is compiled in C modules.  Fortunately, the entire
// header can be wrapped in an 'extern "C"' declaration, and it will then link
// properly.
extern "C" {
#include "zlib.h"
}

#define Z_BUFFER_SIZE 4096  // the size of the input and output buffers
#define Z_USER_ABORT (-7)   // add one error code to those found in zlib.h
//---------------------------------------------------------------------------
// define some new events
typedef void __fastcall (__closure *TZlibProgressEvent)
                        (System::TObject* Sender, int Percent);
typedef void __fastcall (__closure *TZlibResultEvent)
                        (System::TObject* Sender, AnsiString message, int zresult);
//---------------------------------------------------------------------------
class TZlib : public TObject
{
private:
  z_stream *ZStream;   // the zlib compression/decompression structure
  int Percent(void);
  int LoadInput(void);
  int FlushOutput(void);
  TStream *InStream;
  TStream *OutStream;
  long Length;
  int ZlibError;
  unsigned char InputBuffer[Z_BUFFER_SIZE];
  unsigned char OutputBuffer[Z_BUFFER_SIZE];
  bool FUserAbort;
  void __fastcall SetUserAbort(bool value);
  TZlibProgressEvent FOnZlibProgress;
  TZlibResultEvent FOnZlibResult;

protected :
// Derived classes can provide versions of these virtual functions in order to
// customize their program's user interface.  The Abort flag can be set by those
// functions.
  virtual void Progress(int percent);
  virtual void Result(AnsiString message, int zresult);

public :
  __fastcall virtual TZlib(void);
  __fastcall virtual ~TZlib(void);
  int Compress(TStream *input, TStream *output, int level);
  int Decompress(TStream *input, TStream *output);
  __property bool UserAbort = {read=FUserAbort, write=SetUserAbort};
  __property TZlibProgressEvent OnZlibProgress = {read=FOnZlibProgress, write=FOnZlibProgress};
  __property TZlibResultEvent OnZlibResult = {read=FOnZlibResult, write=FOnZlibResult};
};
#endif  // #ifndef TZlibH

